<?php

/**
 * API calls to the Talque backend
 *
 * @link       https://talque.com
 * @since      1.0.0
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/public
 */


/**
 * Custom Exception for API Error
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/public
 * @author     Volker Braun <volker.braun@real-life-interaction.de>
 */
class Talque_Event_API_Error extends Exception {
}


/**
 * API calls to the Talque backend
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/public
 * @author     Volker Braun <volker.braun@real-life-interaction.de>
 */
class Talque_Event_API {

    /**
     * The default value for $backend_url
     */
    const DEFAULT_BACKEND_URL = 'https://event.talque.com';
    
    /**
     * The base url for all external api calls
     */
    private $backend_url;

    /**
     * Initialize the class and set its properties.
     */
    public function __construct() {
        $this->backend_url = get_option(
            'talque-backend', Talque_Event_API::DEFAULT_BACKEND_URL );
        $this->load_dependencies();
    }

    /**
     * Dependency injection
     */
    private function load_dependencies() {
    }

    /**
     * Return the desired locale
     *
     * Either returns 'DE_DE' or 'EN_US', with more locales to be added later.
     */
    private function locale() {
        // type: () -> string
        $locale = get_locale();
        $locale2 = substr( $locale, 0, 2 );  // first two chars
        switch ($locale2) {
            case 'de':
                return 'DE_DE';
            case 'en':
            default:
                return 'EN_US';
        }
    }

    /**
     * Construct absolute url pointing to the backend
     */
    private function url( $local ) {
        // type: (string) -> string
        return $this->backend_url . $local;
    }

    private function url_prefix( $org_id ) {
        $locale = $this->locale();
        if ( ! $org_id ) {
            throw new Talque_Event_API_Error( 'ID not set in the WP plugin options' );
        }
        return "/pub/$locale/org/$org_id";
    }

    private function org_url( $org_local, $date = NULL, $tags = NULL ) {
        // type: (string) -> string
        $params = http_build_query( array(
            'date' => $date,
            'tags' => $tags,
        ) );
        if ( $params )
            $params = '?' . $params;
        return "/$org_local$params";
    }

    private function post_page( $org_url, $shortcode_settings ) {
        // type: (string) -> string
        $settings = Talque_Event_Settings::from_wp_options( $shortcode_settings );
        $prefix = $this->url_prefix( $settings->org_id() );
        $url = $this->url( $prefix . $org_url );
        $args = array(
            'headers' => $settings->headers(),
            'body' => $settings->to_json_encoded(),
        );
        if ( WP_DEBUG ) {
            error_log( 'url = ' . $url );
            error_log( 'body = ' . json_encode( $args ) );
        }
        $response = wp_remote_post( $url, $args );
        $http_code = wp_remote_retrieve_response_code( $response );
        if ( $http_code !== 200 ) {
            error_log( 'backend response: ' . print_r($response, true) );
            throw new Talque_Event_API_Error(
                sprintf('API call failed: %s returned %d', $url, $http_code) );
        }
        return wp_remote_retrieve_body( $response );
    }

    /**
     * Percent-encode the slug, if neccessary
     *
     * This should not be necessary, the slugs are sent percent-encoded from
     * the talque server. But the PHP built-in webserver seems to decode
     * them. A proper lamp server does not decode them.
     *
     * Note that percent characters are not allowed in the (decoded) slug, so we
     * can fix this.
     */
    private function maybe_urlencode( $slug ) {
        $has_percent = ( strpos( $slug, '%' ) !== false );
        if ( $has_percent )
            return $slug;
        // Slug is either decoded or does not need encoding
        return urlencode( $slug );
    }

    public function get_style_url() {
        // type: () -> string
        return $this->url( '/talque-event.css' );
    }
    
    public function get_script_url() {
        // type: () -> string
        return $this->url( '/lib/pub-service.js' );
    }
    
    public function speaker( $speaker_id, $shortcode_settings = NULL ) {
        // type: (string) -> string
        $url = $this->org_url( "speaker/$speaker_id" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function speaker_slug( $speaker_slug, $shortcode_settings = NULL ) {
        // type: (string) -> string
        $slug = $this->maybe_urlencode( $speaker_slug );
        $url = $this->org_url( "speaker-slug/$slug" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function speaker_list( $shortcode_settings = NULL ) {
        $url = $this->org_url( 'speaker-list' );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function speaker_grid( $shortcode_settings = NULL ) {
        // type: () -> string
        $url = $this->org_url( 'speaker-grid' );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function talk( $talk_id, $shortcode_settings = NULL ) {
        // type: (string) -> string
        $url = $this->org_url( "talk/$talk_id" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function talk_slug( $talk_slug, $shortcode_settings = NULL ) {
        // type: (string, Talque_Event_Settings) -> string
        $slug = $this->maybe_urlencode( $talk_slug );
        $url = $this->org_url( "talk-slug/$slug" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function talk_list( $date = NULL, $tags = NULL, $shortcode_settings = NULL ) {
        // type: (?string, ?Talque_Event_Settings) -> string
        $url = $this->org_url( 'talk-list', $date, $tags );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function talk_grid( $date = NULL, $tags = NULL, $shortcode_settings = NULL ) {
        // type: (?string, ?Talque_Event_Settings) -> string
        $url = $this->org_url( 'talk-grid', $date, $tags );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function sponsor( $sponsor_id, $shortcode_settings = NULL ) {
        // type: (string, ?Talque_Event_Settings) -> string
        $url = $this->org_url( "sponsor/$sponsor_id" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function sponsor_slug( $sponsor_slug, $shortcode_settings = NULL ) {
        // type: (string, Talque_Event_Settings) -> string
        $slug = $this->maybe_urlencode( $sponsor_slug );
        $url = $this->org_url( "sponsor-slug/$slug" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function sponsor_grid( $shortcode_settings = NULL ) {
        // type: (?Talque_Event_Settings) -> string
        $url = $this->org_url( 'sponsor-grid' );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function exhibitor( $exhibitor_id, $shortcode_settings = NULL ) {
        // type: (string, ?Talque_Event_Settings) -> string
        $url = $this->org_url( "exhibitor/$exhibitor_id" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function exhibitor_slug( $exhibitor_slug, $shortcode_settings = NULL ) {
        // type: (string, Talque_Event_Settings) -> string
        $slug = $this->maybe_urlencode( $exhibitor_slug );
        $url = $this->org_url( "exhibitor-slug/$slug" );
        return $this->post_page( $url, $shortcode_settings );
    }

    public function exhibitor_grid( $shortcode_settings = NULL ) {
        // type: (?Talque_Event_Settings) -> string
        $url = $this->org_url( 'exhibitor-grid' );
        return $this->post_page( $url, $shortcode_settings );
    }
}
