<?php

/**
 * Custom posts for the talque plugin.
 *
 * @link       https://talque.com
 * @since      1.0.0
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/admin
 */

/**
 * Custom posts for the talque plugin.
 *
 * Defines the plugin name, version, and the custom post types
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/public
 * @author     Volker Braun <volker.braun@real-life-interaction.de>
 */
class Talque_Event_Posts {

    /**
     * The instance
     *
     * @var      Talque_Event_Posts    $instance    The instance this plugin.
     */
    private static $instance;

    /**
     * Get the singleton
     *
     * Use this to access the plugin from templates; When rendering templates,
     * this class has already been instantiated.
     *
     * @return Talque_Event_Posts
     */
    public static function get_instance()
    {
        $inst = self::$instance;
        if ( !$inst )
            throw new Error('the talque event plugin has not been instantiated');
        return $inst;
    }
    
    /**
     * The ID of this plugin.
     *
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Backend API access
     *
     * @var      Talque_Event_API    $instance    The backend API.
     */
    private static $api;

    /**
     * Initialize the class and set its properties.
     *
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version, $api ) {
        // Singleton
        if ( self::$instance ) {
            throw new Error('the talque event plugin can only be instantiated once');
        }
        self::$instance = $this;
        // Initialization
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        self::$api = $api;
        $this->load_dependencies();
    }

    /**
     * Dependency injection
     */
    private function load_dependencies() {
    }

    /**
     * Called by the activator when the plugin is activated
     */
    public function activate_plugin_hook() {
        $this->register_custom_post_type();
        $this->insert_custom_posts();
        $this->add_rewrite_rules();
    }

    /**
     * Called by the deactivator when the plugin is deactivated
     */
    public function deactivate_plugin_hook() {
    }
    
    /**
     * Initialize the plugin
     */
    public function init_hook() {
        $this->register_custom_post_type();
        $this->add_rewrite_rules();
        if ( WP_DEBUG ) {
            // For debugging only: flush now (too expensive in production)
            flush_rewrite_rules();
            error_log( 'flush (WP_DEBUG === true)' );
        }
    }
    
    /**
     * Called when options have been saved
     */
    public function updated_option_hook( $option ) {
        // error_log( 'updated_option_hook: ' . $option );
        if ( in_array( $option, array( 'talque-url-talk', 'talque-url-speaker', 'talque-url-sponsor', 'talque-url-exhibitor' ) ) ) {
            error_log( 'url rewriting options changed, flushing rules' );
            $this->add_rewrite_rules();
            flush_rewrite_rules();
        }
    }

    /**
     * Prevent redirection
     *
     * WPML mucks about with url in a feeble attempt to translate the query; Abort!
     */
    public function wpml_is_redirected_filter( $redirect, $post_id, $q ) {
        // error_log('wpml_is_redirected_filter: '.json_encode($q) );
        if ( isset( $q->query_vars['post_type'] ) && 'talque' === $q->query_vars['post_type'] )
            return false;
        return $redirect;
    }
    
    /**
     * Register the 'talque' custom post type with Wordpress
     *
     * Note: the rewrite rules do not work if Settings -> Permalink Settings
     * -> Plain is selected (which WP treats as "no rewrite rules").
     */
    private function add_rewrite_rules() {
        add_rewrite_tag( '%org-id%', '([0-9a-zA-Z]+)' );
        add_rewrite_tag( '%speaker-id%', '([0-9a-zA-Z]+)' );
        add_rewrite_tag( '%speaker-slug%', '([^/]+)' );
        add_rewrite_tag( '%talk-id%', '([0-9a-zA-Z]+)' );
        add_rewrite_tag( '%talk-slug%', '([^/]+)' );
        add_rewrite_tag( '%sponsor-id%', '([0-9a-zA-Z]+)' );
        add_rewrite_tag( '%exhibitor-id%', '([0-9a-zA-Z]+)' );
        add_rewrite_tag( '%sponsor-slug%', '([^/]+)' );
        add_rewrite_tag( '%exhibitor-slug%', '([^/]+)' );
        add_rewrite_tag( '%date%', '([0-9]+)' );
        # The post_type=talque is not necessary, possibly remnant for supporting old versions?
        add_rewrite_rule( '^talque/sponsors/?$', 'index.php?post_type=talque&talque=sponsor-grid', 'top' );
        add_rewrite_rule( '^talque/sponsor-id/([0-9a-zA-Z]+)?$', 'index.php?post_type=talque&talque=sponsor-grid&sponsor-id=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/sponsor/([^/]+)/?$', 'index.php?post_type=talque&talque=sponsor-grid&sponsor-slug=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/exhibitors/?$', 'index.php?post_type=talque&talque=exhibitor-grid', 'top' );
        add_rewrite_rule( '^talque/exhibitor-id/([0-9a-zA-Z]+)?$', 'index.php?post_type=talque&talque=exhibitor-grid&exhibitor-id=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/exhibitor/([^/]+)/?$', 'index.php?post_type=talque&talque=exhibitor-grid&exhibitor-slug=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/speaker-grid/?$', 'index.php?post_type=talque&talque=speaker-grid', 'top' );
        add_rewrite_rule( '^talque/speaker-list/?$', 'index.php?post_type=talque&talque=speaker-list', 'top' );
        add_rewrite_rule( '^talque/speaker-id/([0-9a-zA-Z]+)/?$', 'index.php?post_type=talque&talque=speaker-list&speaker-id=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/speaker/([^/]+)/?$', 'index.php?post_type=talque&talque=speaker-list&speaker-slug=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/talk-grid/?$', 'index.php?post_type=talque&talque=talk-grid', 'top' );
        add_rewrite_rule( '^talque/talk-list/?$', 'index.php?post_type=talque&talque=talk-list', 'top' );
        add_rewrite_rule( '^talque/talk-id/([0-9a-zA-Z]+)/?$', 'index.php?post_type=talque&talque=talk-list&talk-id=$matches[1]', 'top' );
        add_rewrite_rule( '^talque/talk/([^/]+)/?$', 'index.php?post_type=talque&talque=talk-list&talk-slug=$matches[1]', 'top' );
        // Customizable urls
        $speaker = get_option( 'talque-url-speaker' );
        if ( $speaker )
            add_rewrite_rule( '^' . $speaker . '/([^/]+)/?$', 'index.php?post_type=talque&talque=speaker-list&speaker-slug=$matches[1]', 'top' );
        $talk = get_option( 'talque-url-talk' );
        if ( $talk )
            add_rewrite_rule( '^' . $talk . '/([^/]+)/?$', 'index.php?post_type=talque&talque=talk-list&talk-slug=$matches[1]', 'top' );
        $sponsor = get_option( 'talque-url-sponsor' );
        if ( $sponsor )
            add_rewrite_rule( '^' . $sponsor . '/([^/]+)/?$', 'index.php?post_type=talque&talque=sponsor-grid&sponsor-slug=$matches[1]', 'top' );
        $exhibitor = get_option( 'talque-url-exhibitor' );
        if ( $exhibitor )
            add_rewrite_rule( '^' . $exhibitor . '/([^/]+)/?$', 'index.php?post_type=talque&talque=exhibitor-grid&exhibitor-slug=$matches[1]', 'top' );
    }

    /**
     * Register the stylesheet and JavaScript for the custom post type
     */
    public function wp_enqueue_scripts_hook() {
        wp_enqueue_style(
            $this->plugin_name,
            self::$api->get_style_url(),
            array(), $this->version, 'all' );
        wp_enqueue_script(
            $this->plugin_name,
            self::$api->get_script_url(),
            array(), $this->version, false );
    }
    
    /**
     * Extends the single_template filter to find the custom post template
     */
    public function single_template_filter( $template ) {
        // type: (string) -> string
        $post_type = get_query_var( 'post_type' );
        if ($post_type != 'talque')
            return $template;
        $plugin = plugin_dir_path( dirname( __FILE__ ) );
        return $plugin . '/public/single-talque.php';
    }

    /**
     * Register the 'talque' custom post type with Wordpress
     */
    private function register_custom_post_type() {
        register_post_type(
            'talque',
            array(
                'labels'      => array(
                    'name'        => __('Talque'),
                ),
                'public'      => true,
                'has_archive' => false,
                'show_ui'     => false,
                'show_in_menu' => false,
                'show_in_nav_menus' => true,
            )
        );
    }
    
    /**
     * Insert the 'talque' custom posts
     */
    private function insert_custom_posts() {
        $speaker_list = $this->get_or_insert_post( 'speaker-list', __('Speaker List') );
        $speaker_grid = $this->get_or_insert_post( 'speaker-grid', __('Speaker Grid') );
        $talk_list = $this->get_or_insert_post( 'talk-list', __('Talk List') );
        $talk_grid = $this->get_or_insert_post( 'talk-grid', __('Talk Grid') );
        $sponsor_grid = $this->get_or_insert_post( 'sponsor-grid', __( 'Sponsors' ) );
        $exhibitor_grid = $this->get_or_insert_post( 'exhibitor-grid', __( 'Exhibitors' ) );
        error_log(
            'talque posts: ' . 
            $speaker_list . '/' .
            $speaker_grid . '/' .
            $talk_list . '/' .
            $talk_grid . '/' .
            $sponsor_grid . '/' .
            $exhibitor_grid
        );
    }

    private function get_or_insert_post( $slug, $title ) {
        // type: (string, string) -> int
        $post = get_page_by_title( $title, OBJECT, 'talque' );
        if ( $post )
            $ID = $post->ID;
        else
            $ID = NULL;
        $post_id = wp_insert_post( array(
            'ID' => $ID,
            'post_type' => 'talque',
            'post_name' => $slug,
            'post_title' => $title,
            'post_status' => 'publish',
        ) );
        return $post_id;
    }

    /**
     * Return appropriate content by making a backend API call
     *
     * Throws if an error is encountered.
     */
    public function get_content( $slug ) {
        // type: (string) -> string
        $date = get_query_var( 'date', NULL );
        if ( WP_DEBUG )
            error_log("get_content $slug " .
                      ' slug=' . get_query_var( 'speaker-slug' ) .
                      ' org-id=' . get_query_var( 'org-id' ));
        $settings = Talque_Event_Settings::from_org_id( get_query_var( 'org-id', NULL ) );
        switch ( $slug ) {
            case 'speaker-list':
                $speaker_id = get_query_var( 'speaker-id' );
                if ( $speaker_id ) {
                    return self::$api->speaker( $speaker_id, $settings );
                }
                $speaker_slug = get_query_var( 'speaker-slug' );
                if ( $speaker_slug ) {
                    return self::$api->speaker_slug( $speaker_slug, $settings );
                }
                return self::$api->speaker_list();
            case 'speaker-grid':
                return self::$api->speaker_grid();
            case 'talk-list':
                $talk_id = get_query_var( 'talk-id' );
                if ( $talk_id ) {
                    return self::$api->talk( $talk_id, $settings );
                }
                $talk_slug = get_query_var( 'talk-slug' );
                if ( $talk_slug ) {
                    return self::$api->talk_slug( $talk_slug, $settings );
                }
                return self::$api->talk_list( $date );
            case 'talk-grid':
                return self::$api->talk_grid( $date );
            case 'sponsor-grid':
                $sponsor_id = get_query_var( 'sponsor-id' );
                if ( $sponsor_id ) {
                    return self::$api->sponsor( $sponsor_id );
                }
                $sponsor_slug = get_query_var( 'sponsor-slug' );
                if ( $sponsor_slug ) {
                    return self::$api->sponsor_slug( $sponsor_slug, $settings );
                }
                return self::$api->sponsor_grid();
            case 'exhibitor-grid':
                $exhibitor_id = get_query_var( 'exhibitor-id' );
                if ( $exhibitor_id ) {
                    return self::$api->exhibitor( $exhibitor_id, $settings );
                }
                $exhibitor_slug = get_query_var( 'exhibitor-slug' );
                if ( $exhibitor_slug ) {
                    return self::$api->exhibitor_slug( $exhibitor_slug, $settings );
                }
                return self::$api->exhibitor_grid();
            default:
                throw new Error( 'not handled: ' . $slug );
        }
    }

    /**
     * Return content or an error message
     */
    public function get_content_or_error( $slug ) {
        // type: (string) -> string
        try {
            return $this->get_content( $slug );
        } catch ( Talque_Event_API_Error $error ) {
            return $error->getMessage();
        }
    }
}

