<?php

/**
 * Settings for customizing the appearence of the Talque backend
 *
 * @link       https://talque.com
 * @since      1.0.0
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/public
 */


/**
 * Settings for customizing the appearence of the Talque backend
 *
 * @package    Talque_Event
 * @subpackage Talque_Event/public
 * @author     Volker Braun <volker.braun@real-life-interaction.de>
 */
class Talque_Event_Settings {

    private $json;

    /**
     * Initialize the class and set its properties.
     */
    public function __construct($json) {
        $this->json = $json;
        // error_log( 'Talque_Event_Settings: ' . json_encode($this->json) );
    }

    private static function set_optional( $key, $array, $source_key, $source_array ) {
        // Note: shortcode attrs are lower cased
        $source_key_lc = strtolower( $source_key );
        $value = $source_array[$source_key_lc];
        if ( $value !== NULL )
            $array[$key] = $value;
    }

    private static function set_optional_wp( $name, $key, $array, $override_array=NULL ) {
        if ( $override_array->offsetExists( $key ) ) {
            $array[$key] = $override_array[$key];
            return;
        }
        $value = get_option( $name );
        if ( ! $value )
            return;
        $array[$key] = $value;
    }

    public static function make_empty() {
        $palette = new ArrayObject();
        $padding = new ArrayObject();
        $style = new ArrayObject();
        $style['palette'] = $palette;
        $style['padding'] = $padding;
        $route = new ArrayObject();
        $route['embed'] = 'wordpress';
        $json = new ArrayObject();
        $json['style'] = $style;
        $json['route'] = $route;
        return new Talque_Event_Settings( $json );
    }

    public static function from_org_id( $org_id ) {
        $attrs = array(
            'orgid' => $org_id
        );
        return self::from_shortcode( $attrs );
    }
    
    public static function from_shortcode( $attrs ) {
        // Note: shortcode attrs are lower cased
        $sanitized = shortcode_atts( array(
            'orgid' => NULL,
            'primary' => NULL,
            'accent' => NULL,
            'speakerphoto' => NULL,
            'timetic' => NULL,
            'align' => NULL,
            'depth' => NULL,
            'backgroundcolor' => NULL,
            'maxwidth' => NULL,
            'padding' => NULL,
            'mobilepadding' => NULL,
        ), $attrs );
        $palette = new ArrayObject();
        self::set_optional( 'primary', $palette, 'primary', $sanitized );
        self::set_optional( 'accent', $palette, 'accent', $sanitized );
        $padding = new ArrayObject();
        self::set_optional( 'desktop', $padding, 'padding', $sanitized );
        self::set_optional( 'mobile', $padding, 'mobilePadding', $sanitized );
        $style = new ArrayObject();
        $style['palette'] = $palette;
        $style['padding'] = $padding;
        self::set_optional( 'speakerPhoto', $style, 'speakerPhoto', $sanitized );
        self::set_optional( 'timeTic', $style, 'timeTic', $sanitized );
        self::set_optional( 'align', $style, 'align', $sanitized );
        self::set_optional( 'depth', $style, 'depth', $sanitized );
        self::set_optional( 'backgroundColor', $style, 'backgroundColor', $sanitized );
        self::set_optional( 'maxwidth', $style, 'maxwidth', $sanitized );
        $route = new ArrayObject();
        $route['embed'] = 'wordpress';
        self::set_optional( 'orgId', $route, 'orgId', $sanitized );
        $json = new ArrayObject();
        $json['style'] = $style;
        $json['route'] = $route;
        return new Talque_Event_Settings( $json );
    }
    
    public static function from_wp_options( $override = NULL ) {
        if ( $override === NULL )
            $o = self::make_empty()->json;
        else
            $o = $override->json;
        $palette = new ArrayObject();
        self::set_optional_wp( 'talque-palette-primary', 'primary', $palette, $o['style']['palette'] );
        self::set_optional_wp( 'talque-palette-accent', 'accent', $palette, $o['style']['palette'] );
        $padding = new ArrayObject();
        self::set_optional_wp( 'talque-padding', 'desktop', $padding, $o['style']['padding'] );
        self::set_optional_wp( 'talque-padding-mobile', 'mobile', $padding, $o['style']['padding'] );
        $style = new ArrayObject();
        $style['palette'] = $palette;
        $style['padding'] = $padding;
        self::set_optional_wp( 'talque-speakerPhoto', 'speakerPhoto', $style, $o['style'] );
        self::set_optional_wp( 'talque-timeTic', 'timeTic', $style, $o['style'] );
        self::set_optional_wp( 'talque-align', 'align', $style, $o['style'] );
        self::set_optional_wp( 'talque-depth', 'depth', $style, $o['style'] );
        self::set_optional_wp( 'talque-backgroundColor', 'backgroundColor', $style, $o['style'] );
        self::set_optional_wp( 'talque-maxwidth', 'maxwidth', $style, $o['style'] );
        $route = new ArrayObject();
        $route['embed'] = 'wordpress';
        self::set_optional_wp( 'talque-org-id', 'orgId', $route, $o['route'] );
        if ( !isset( $o['route']['orgId'] ) ) {
            // Only allow speaking urls when not overriding the org
            self::set_optional_wp( 'talque-url-speaker', 'speakerUrl', $route, $o['route'] );
            self::set_optional_wp( 'talque-url-talk', 'talkUrl', $route, $o['route'] );
            self::set_optional_wp( 'talque-url-sponsor', 'sponsorUrl', $route, $o['route'] );
            self::set_optional_wp( 'talque-url-exhibitor', 'exhibitorUrl', $route, $o['route'] );
        }
        $json = new ArrayObject();
        $json['style'] = $style;
        $json['route'] = $route;
        return new Talque_Event_Settings( $json );
    }

    public function to_json() {
        return $this->json;
    }
                    
    public function to_json_encoded() {
        return json_encode( $this->to_json() );
    }

    private function speaker_url() {
        $route = $this->json['route'];
        if ( isset( $route['speakerUrl'] ) )
            return $route['speakerUrl'];
        return NULL;
    }
    
    private function talk_url() {
        $route = $this->json['route'];
        if ( isset( $route['talkUrl'] ) )
            return $route['talkUrl'];
        return NULL;
    }

    private function sponsor_url() {
        $route = $this->json['route'];
        if ( isset( $route['sponsorUrl'] ) )
            return $route['sponsorUrl'];
        return NULL;
    }

    private function exhibitor_url() {
        $route = $this->json['route'];
        if ( isset( $route['exhibitorUrl'] ) )
            return $route['exhibitorUrl'];
        return NULL;
    }

    public function org_id() {
        $route = $this->json['route'];
        if ( isset( $route['orgId'] ) )
            return $route['orgId'];
        return NULL;
    }

    public function headers()  {
        return array(
            'Talque-Embed' => 'wordpress',
            'Talque-SiteUrl' => get_home_url(),
            'Talque-Speaker' => $this->speaker_url(),
            'Talque-Talk' => $this->talk_url(),
            'Talque-Sponsor' => $this->sponsor_url(),
            'Talque-Exhibitor' => $this->exhibitor_url(),
        );
    }
}
